@@ -9,11 +9,11 @@ Slackware::Slackget::File - A class to manage files.
 
 =head1 VERSION
 
-Version 1.0.3
+Version 1.0.5
 
 =cut
 
-our $VERSION = '1.0.3';
+our $VERSION = '1.0.5';
 
 =head1 SYNOPSIS
 
@@ -33,6 +33,10 @@ Access to hard disk are saved by taking a copy of the file in memory, so if you
 
 The main advantage of this module is that you don't work directly on the file but on a copy. So you can make errors, they won't be wrote until you call the Write() method
 
+** ATTENTION ** this module can fail to load file on non-UNIX system because it rely on the "file" and "awk" command line tools. Be sure to use the 'load-raw' => 1 constructor's option on such operating system (most probably the file type will be blank and no problem will happen... but it's still a possibility).
+
+** ATTENTION 2 ** this module rely on bzip2 and gzip command line tools to uncompress the compressed files. On systems which does not support the `gzip -dc` or `bzip2 -dc`, trying to load compressed files will cause crashs, which can eventually lead to the end of the world...
+
 =cut
 
 sub new
@@ -44,10 +48,24 @@ sub new
 	$self->{'file-encoding'} = 'utf8' unless(defined($self->{'file-encoding'}));
 	if(defined($file) && -e $file && !defined($args{'load-raw'}))
 	{
-		$self->{TYPE} = `LC_ALL=C file -b $file | awk '{print \$1}'`;
-		chomp $self->{TYPE};
+		eval {
+			$self->{TYPE} = `LC_ALL=C file -b $file | awk '{print \$1}'`;
+			chomp $self->{TYPE};
+		};
+		if($@){
+			$self->{TYPE} = `LC_ALL=C file $file | awk '{print \$2}'`;
+			chomp $self->{TYPE};
+			if($@){
+				$self->{TYPE} = 'none' ; # Empty the file type if the `file` syscall failed
+				$args{'load-raw'}=1; # and set option to load it raw
+			}
+		}
+		$self->{TYPE} = 'none' unless(defined($self->{TYPE}));
 		$self->{TYPE} = 'ASCII' if($self->{TYPE} eq 'empty');
-		die "[Slackware::Slackget::File::constructor] unsupported file type \"$self->{TYPE}\" for file $file. Supported file type are gzip, bzip2, ASCII and XML\n" unless($self->{TYPE} eq 'gzip' || $self->{TYPE} eq 'bzip2' || $self->{TYPE} eq 'ASCII' || $self->{TYPE} eq 'XML' || $self->{TYPE} eq 'Quake') ;
+		$self->{TYPE} = 'ASCII' if($self->{TYPE} eq 'XML' || $self->{TYPE} eq 'Quake');
+		die "[Slackware::Slackget::File::constructor] unsupported file type \"$self->{TYPE}\" for file $file. Supported file type are gzip, bzip2, ASCII and XML\n" unless($self->{TYPE} eq 'gzip' || $self->{TYPE} eq 'bzip2' || $self->{TYPE} eq 'ASCII' || $self->{TYPE} eq 'XML' || $self->{TYPE} eq 'none') ;
+	}else{
+		$self->{TYPE} = 'ASCII' ;
 	}
 # 	print "using $self->{'file-encoding'} as file-encoding for file $file\n";
 	$self->{FILENAME} = $file;
@@ -57,7 +75,7 @@ sub new
 	$self->{BINARY} = $args{'binary'} if($args{'binary'});
 	$self->{SKIP_WL} = $args{'skip-white-line'} if($args{'skip-white-line'});
 	$self->{SKIP_WL} = $args{'skip-white-lines'} if($args{'skip-white-lines'});
-	$self->{LOAD_RAW} = undef;
+	$self->{LOAD_RAW} = 0;
 	$self->{LOAD_RAW} = $args{'load-raw'} if($args{'load-raw'});
 	if(defined($file) && -e $file && !defined($self->{'no-auto-load'})){
 		$self->Read();
@@ -145,7 +163,7 @@ sub Read
 	}
 	my $tmp;
 	my @file = ();
-	if(defined($self->{TYPE}) && ($self->{TYPE} eq 'ASCII' || $self->{TYPE} eq 'XML' || $self->{TYPE} eq 'Quake') && !defined($self->{LOAD_RAW}))
+	if((defined($self->{TYPE}) && ($self->{TYPE} eq 'ASCII' || $self->{TYPE} eq 'XML' || $self->{TYPE} eq 'Quake') ) && !$self->{LOAD_RAW})
 	{
 # 		print "[DEBUG] [Slackware::Slackget::File] loading $file as 'plain text' file.";
 		if(open (F2,"<:encoding($self->{'file-encoding'})",$file))
@@ -178,7 +196,7 @@ sub Read
 			return undef;
 		}
 	}
-	elsif($self->{TYPE} eq 'bzip2' && !defined($self->{LOAD_RAW}))
+	elsif($self->{TYPE} eq 'bzip2' && !$self->{LOAD_RAW})
 	{
 # 		print "[DEBUG] [Slackware::Slackget::File] loading $file as 'bzip2' file.";
 # 		my $tmp_file = `bzip2 -dc $file`;
@@ -189,7 +207,7 @@ sub Read
 		$self->{FILE} = \@file ;
 		return 1;
 	}
-	elsif($self->{TYPE} eq 'gzip' && !defined($self->{LOAD_RAW}))
+	elsif($self->{TYPE} eq 'gzip' && !$self->{LOAD_RAW})
 	{
 # 		print "[DEBUG] [Slackware::Slackget::File] loading $file as 'gzip' file.";
 # 		my $tmp_file = `gzip -dc $file`;
@@ -200,7 +218,7 @@ sub Read
 		$self->{FILE} = \@file ;
 		return 1;
 	}
-	elsif(defined($self->{LOAD_RAW}) or $self->{TYPE} eq '')
+	elsif($self->{LOAD_RAW} or $self->{TYPE} eq '')
 	{
 # 		print "[DEBUG] [Slackware::Slackget::File] loading $file as 'raw' file.";
 		if(open(F2,$file))
@@ -227,12 +245,22 @@ sub Read
 
 }
 
-=head2 Lock_file
+=head2 Lock_file (deprecated)
+
+Same as lock_file, provided for backward compatibility.
+
+=cut
+
+sub Lock_file {
+	return lock_file(@_);
+}
+
+=head2 lock_file
 
 This method lock the file for slack-get application (not really for others...) by creating a file with the name of the current open file plus a ".lock". This is not a protection but an information system for slack-getd sub process. This method return undef if the lock can't be made.
 
 	my $file = new Slackware::Slackget::File ('test.txt');
-	$file->Lock_file ; # create a file test.txt.lock
+	$file->lock_file ; # create a file test.txt.lock
 
 ATTENTION: You can only lock the current file of the object. With the previous example you can't do :
 
@@ -242,7 +270,7 @@ ATTENTION 2 : Don't forget to unlock your locked file :)
 
 =cut
 
-sub Lock_file
+sub lock_file
 {
 	my $self = shift;
 	return undef if $self->is_locked ;
@@ -251,11 +279,21 @@ sub Lock_file
 	return 1;
 }
 
-=head2 Unlock_file
+=head2 Unlock_file (deprecated)
+
+Same as unlock_file(), provided for backward compatibility.
+
+=cut
+
+sub Unlock_file {
+	return unlock_file(@_);
+}
+
+=head2 unlock_file
 
 Unlock a locked file. Only the locker object can unlock a file ! Return 1 if all goes well, else return undef. Return 2 if the file was not locked. Return 0 (false in scalar context) if the file was locked but by another Slackware::Slackget::File object.
 
-	my $status = $file->Unlock_file ;
+	my $status = $file->unlock_file ;
 
 Returned value are :
 
@@ -269,7 +307,7 @@ Returned value are :
 
 =cut
 
-sub Unlock_file
+sub unlock_file
 {
 	my $self = shift;
 	if($self->is_locked)
@@ -295,7 +333,7 @@ sub _verify_lock_maker
 {
 	my $self = shift;
 	my $file = new Slackware::Slackget::File ("$self->{FILENAME}.lock");
-	my $locker = $file->Get_line(0) ;
+	my $locker = $file->get_line(0) ;
 # 	print "\t[DEBUG] ( Slackware::Slackget::File in _verify_lock_maker() ) locker of file \"$self->{FILENAME}\" is $locker and current object is $self\n";
 	$file->Close ;
 	undef($file);
@@ -363,7 +401,7 @@ sub Write
 	if(open (FILE, "$mode:encoding($self->{'file-encoding'})",$name))
 	{
 		binmode(FILE) if($self->{'BINARY'}) ;
-		# NOTE: In the case you need to clear the white line of your file, their will be a if() test no each array slot
+		# NOTE: In the case you need to clear the white line of your file, their will be a if() test for each array slot
 		# This is really time consumming, so id you don't need this feature we just test once for all and gain a lot in performance.
 		if($self->{SKIP_WL})
 		{
@@ -394,60 +432,102 @@ sub Write
 	return 1;
 }
 
-=head2 Add
+=head2 Add (deprecated)
 
-Take a table of lines and add them to the end of file image (in memory). You need to commit your change by calling the Write() method !
-
-	$file->Add(@data);
-	or
-	$file->Add($data);
-	or
-	$file->Add("this is some data\n");
+Same as add(), provided for backward compatibility.
 
 =cut
 
 sub Add {
+	return add(@_);
+}
+
+=head2 add
+
+Take a table of lines and add them to the end of file image (in memory). You need to commit your change by calling the Write() method !
+
+	$file->add(@data);
+	or
+	$file->add($data);
+	or
+	$file->add("this is some data\n");
+
+=cut
+
+sub add {
 	my ($self,@data) = @_;
 	$self->{FILE} = [@{$self->{FILE}},@data];
 }
 
-=head2 Get_file
+=head2 Get_file (deprecated)
 
-Return the current file in memory as an array.
-
-	@file = $file->Get_file();
+Same as get_file(), provided for backward compatibility.
 
 =cut
 
-sub Get_file{
+sub Get_file {
+	return get_file(@_);
+}
+
+=head2 get_file
+
+Return the current file in memory as an array.
+
+	@file = $file->get_file();
+
+=cut
+
+sub get_file{
 	my $self = shift;
 	return @{$self->{FILE}};
 }
 
-=head2 Get_line
 
-Return the $index line of the file (the index start at 0).
+=head2 Get_line (deprecated)
 
-	@file = $file->Get_line($index);
+Same as get_line(), provided for backward compatibility.
 
 =cut
 
 sub Get_line {
+	return get_line(@_);
+}
+
+
+=head2 get_line
+
+Return the $index line of the file (the index start at 0).
+
+	@file = $file->get_line($index);
+
+=cut
+
+sub get_line {
 	my ($self,$index) = @_;
 	return $self->{FILE}->[$index];
 }
 
-=head2 Get_selection
+=head2 Get_selection (deprecated)
+
+Same as get_selection(), provided for backward compatibility.
+
+=cut
+
+sub Get_selection {
+	return get_selection(@_);
+}
+
+=head2 get_selection
 
 	Same as get file but return only lines between $start and $stop.
 
-	my @array = $file->Get_selection($start,$stop);
+	my @array = $file->get_selection($start,$stop);
 
 You can ommit the $stop parameter (in this case Get_line() return the lines from $start to the end of file)
 
 =cut
 
-sub Get_selection {
+sub get_selection {
 	my ($self,$start,$stop) = @_ ;
 	$start = 0 unless($start);
 	$stop = $#{$self->{FILE}} unless($stop);
@@ -466,19 +546,31 @@ Free the memory. This method close the current file memory image. If you don't c
 sub Close {
 	my $self = shift;
 	$self->{FILE} = [];
+	return 1;
 }
 
-=head2 Write_and_close
 
-An alias which call Write() and then Close();
+=head2 Write_and_close (deprecated)
 
-	$file->Write_and_close();
-	or
-	$file->Write_and_close("foo.txt");
+Same as write_and_close(), provided for backward compatibility.
 
 =cut
 
-sub Write_and_close{
+sub Write_and_close {
+	return write_and_close(@_);
+}
+
+=head2 write_and_close
+
+An alias which call Write() and then Close();
+
+	$file->write_and_close();
+	or
+	$file->write_and_close("foo.txt");
+
+=cut
+
+sub write_and_close{
 	my ($self,$file) = @_;
 	$self->Write($file);
 	$self->Close();
@@ -496,16 +588,6 @@ Without parameter return the current file encoding, with a parameter set the enc
 sub encoding
 {
 	return $_[1] ? $_[0]->{'file-encoding'}=$_[1] : $_[0]->{'file-encoding'};
-# 	Idem au code suivant (qui est vachement plus clair mais aussi beeeeaaaaauuuucoup plus long), on economise ici 2 variables locales : meilleurs perf.
-# 	my ($self,$encoding) = @_;
-# 	if(defined($encoding))
-# 	{
-# 		$self->{'file-encoding'} = $encoding ;
-# 	}
-# 	else
-# 	{
-# 		return $self->{'file-encoding'};
-# 	}
 }
 
 =head2 filename
@@ -514,11 +596,27 @@ Return the filename of the file which is currently process by the Slackware::Sla
 
 	print $file->filename
 
+You can also set the filename :
+
+	$file->filename('foo.txt');
+
 =cut
 
 sub filename
 {
-	return $_[0]->{FILENAME} ;
+	return $_[1] ? $_[0]->{FILENAME}=$_[1] : $_[0]->{FILENAME};
+}
+
+=head2 type (read only)
+
+Return the current file type.
+
+	print $file->type
+
+=cut
+
+sub type {
+	return $_[0]->{TYPE};
 }
 
 =head1 AUTHOR
@@ -537,7 +635,7 @@ your bug as I make changes.
 
 You can find documentation for this module with the perldoc command.
 
-    perldoc Slackware::Slackget
+    perldoc Slackware::Slackget::File
 
 
 You can also look for information at:
@@ -546,7 +644,7 @@ You can also look for information at:
 
 =item * Infinity Perl website
 
-L<http://www.infinityperl.org>
+L<http://www.infinityperl.org/category/slack-get>
 
 =item * slack-get specific website
 
