@@ -65,6 +65,8 @@ sub new
 
 This class inheritate from Slackware::Slackget::List (L<Slackware::Slackget::List>), so you may want read the Slackware::Slackget::List documentation for the supported methods of this class.
 
+The present documentation present only methods that differs from the Slackware::Slackget::List class.
+
 =cut
 
 =head2 fill_from_xml
@@ -173,6 +175,7 @@ sub index_list
 	$self->{INDEX} = {} ;
 	foreach my $pkg (@{$self->{LIST}})
 	{
+# 		print "[Slackware::Slackget::PackageList] indexing package: ",$pkg->get_id(),"\n";
 		$self->{INDEX}->{$pkg->get_id()} = $pkg ;
 	}
 	return 1;
@@ -241,7 +244,7 @@ You can also look for information at:
 
 =item * Infinity Perl website
 
-L<http://www.infinityperl.org>
+L<http://www.infinityperl.org/category/slack-get>
 
 =item * slack-get specific website
 
